@@ -689,13 +689,14 @@ sub inttypesDefault() {
 
         my $works;
 
-        # We have a report (2005.09.17) that on IRIX 5.3 with the
-        # native IDO cc, inttypes.h and sys/types.h conflict (and
-        # Netpbm program include sys/types for otherthings), so we
-        # prefer sys/types.h to inttypes.h here.
+        # We saw a system (Irix 5.3 with native IDO, December 2005) on
+        # which sys/types.h defines uint32_t, but not int32_t and other
+        # similar types.  We saw a Mac OS X system (January 2006) on which
+        # sys/types sometimes defines uint32_t, but sometimes doesn't.
+        # So we make those last resorts.
 
-        my @candidateList = ("<types.h>", "<sys/types.h>",
-                             "<inttypes.h>", "<sys/inttypes.h>");
+        my @candidateList = ("<inttypes.h>", "<sys/inttypes.h>",
+                             "<types.h>", "<sys/types.h>");
         
         for (my $i = 0; $i < @candidateList && !$works; ++$i) {
             my $candidate = $candidateList[$i];
@@ -1031,6 +1032,8 @@ sub getLinuxsvgaLibrary($) {
         if (defined($linuxsvgahdr_dir)) {
             if (-f "/usr/lib/libvga.so") {
                 $linuxsvgalib = "/usr/lib/libvga.so";
+            } elsif (-f "/usr/lib/svgalib/libvga.so") {
+                $linuxsvgalib = "/usr/lib/svgalib/libvga.so";
             } elsif (-f "/usr/link/svgalib/libvga.so") {
                 $linuxsvgalib = "/usr/link/svgalib/libvga.so";
             } else {
