@@ -22,11 +22,9 @@
 #include <limits.h>
 #include <assert.h>
 
-#include "pm_c_util.h"
-#include "shhopt.h"
-#include "nstring.h"
-#include "bitio.h"
 #include "pnm.h"
+#include "shhopt.h"
+
 #include "bmp.h"
 
 /* MAXCOLORS is the maximum size of a color map in a BMP image */
@@ -88,7 +86,11 @@ struct bmpInfoHeader {
         */
     int cmapsize;
         /* Size in bytes of the colormap (palette) in the BMP file */
+    unsigned int imageSize;
+        /* Size in bytes of the image data.  We only reference this 
+           when the image is compressed. */    
     unsigned short cPlanes;
+    unsigned long int compression;
     struct pixelformat pixelformat;
 };
 
@@ -102,7 +104,7 @@ struct cmdline_info {
     int verbose;    /* -verbose option */
 };
 
-static const char * ifname;
+static const char *ifname;
 
 
 
@@ -122,7 +124,7 @@ parse_command_line(int argc, char ** argv,
 
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENTRY(0,   "verbose",     OPT_FLAG,   &cmdline_p->verbose,         0);
-
+ 
     /* Set the defaults */
     cmdline_p->verbose = FALSE;
 
@@ -171,6 +173,17 @@ GetShort(FILE * const fp) {
     return v;
 }
 
+static short
+GetBigShort(FILE * const fp) {
+
+    short           v;
+
+    if (pm_readbigshort(fp, &v) == -1)
+        pm_error(er_read, ifname);
+
+    return v;
+}
+
 
 
 static long
@@ -223,26 +236,25 @@ readOffBytes(FILE * const fp, unsigned int const nbytes) {
 
 
 static void
-BMPreadfileheader(FILE *         const ifP, 
+BMPreadfileheader(FILE * const ifP, 
                   unsigned int * const bytesReadP, 
                   unsigned int * const offBitsP) {
 
-    unsigned long   cbSize;
     unsigned short  xHotSpot;
     unsigned short  yHotSpot;
     unsigned long   offBits;
+    unsigned long int fileSize;
+
 
     if (GetByte(ifP) != 'B')
-        pm_error("'%s' is not a BMP file.  (It doesn't start with 'BM')",
-                 ifname);
+        pm_error("%s is not a BMP file", ifname);
     if (GetByte(ifP) != 'M')
-        pm_error("'%s' is not a BMP file.  (It doesn't start with 'BM')",
-                 ifname);
+        pm_error("%s is not a BMP file", ifname);
 
-    cbSize = GetLong(ifP);
-    xHotSpot = GetShort(ifP);
-    yHotSpot = GetShort(ifP);
-    offBits = GetLong(ifP);
+    fileSize = GetLong(ifP);  /* This is not always reliable. */
+    xHotSpot  = GetShort(ifP);
+    yHotSpot  = GetShort(ifP);
+    offBits   = GetLong(ifP);
 
     *offBitsP = offBits;
 
@@ -278,7 +290,9 @@ readOs2InfoHeader(FILE *                 const ifP,
     else
         pm_error("Unrecognized bits per pixel in OS/2 BMP file header: %d",
                  headerP->cBitCount);
-
+                 
+    headerP->compression = COMP_RGB;
+    
     pm_message("OS/2 BMP, %dx%dx%d",
                headerP->cols, headerP->rows, headerP->cBitCount);
 }
@@ -315,17 +329,30 @@ readWindowsBasic40ByteInfoHeader(FILE *                 const ifP,
     headerP->cBitCount = GetShort(ifP);
  
     {
-        unsigned long const compression = GetLong(ifP);
+        unsigned long int const compression = GetLong(ifP);
 
         headerP->bitFields = (compression == COMP_BITFIELDS);
-            
-        if (compression != COMP_RGB && compression != COMP_BITFIELDS) 
-            pm_error("Input is compressed.  This program doesn't handle "
-                     "compressed images.  Compression type code = %ld",
-                     compression);
+    
+        if (compression != COMP_RGB && compression != COMP_BITFIELDS &&
+            compression != COMP_RLE4 && compression != COMP_RLE8 ) 
+            pm_error("Input is compressed.  Unsupported encoding method.\n"
+                     "Compression type code = %ld", compression);
+                     
+        if ( (compression == COMP_RLE4 || compression == COMP_RLE8) &&
+              headerP->rowOrder == TOPDOWN )                        
+            pm_error("Invalid BMP header.  Top-down images cannot be compressed.");
+
+        if ( (compression == COMP_RLE4 && headerP->cBitCount !=4) ||
+             (compression == COMP_RLE8 && headerP->cBitCount !=8) )                        
+            pm_error("Invalid BMP header.\n" 
+                     "Compression type (%s) disagrees with number of bits per pixel (%u).",
+                      compression == COMP_RLE4 ? "RLE4" : "RLE8",
+                      headerP->cBitCount);
+
+        headerP->compression = compression;             
     }
     /* And read the rest of the junk in the 40 byte header */
-    GetLong(ifP);   /* ImageSize */
+    headerP->imageSize = GetLong(ifP);   /* ImageSize */
     GetLong(ifP);   /* XpixelsPerMeter */
     GetLong(ifP);   /* YpixelsPerMeter */
     colorsused = GetLong(ifP);   /* ColorsUsed */
@@ -356,21 +383,29 @@ readWindowsBasic40ByteInfoHeader(FILE *                 const ifP,
 
 
 static unsigned int
-lsbZeroCount(unsigned int const mask) {
+lsbZeroCount(unsigned int const mask)
 /*----------------------------------------------------------------------------
    Return the number of consecutive zeroes in the mask 'mask', starting with
-   the least significant bit and going up.  E.g. for 0x1F, it would be 5.
+   the least significant bit and going up.  E.g. for 0x20, it would be 5.
+   
+   Use GCC built-in when available.
 -----------------------------------------------------------------------------*/
-    unsigned int i;
 
-    i = 0;
+#if ( defined(__GNUC__) && (__GNUC__ * 100 + __GNUC_MINOR__ >= 304) )
+{
+      return ( mask==0 ? sizeof(mask)*8 : __builtin_ctz(mask) );
+}
+#else
+{
+      unsigned int i=0;
     
-    while (((mask >> i) & 0x1) == 0 && i < sizeof(mask)*8)
+      while (((mask >> i) & 0x1) == 0 && i < sizeof(mask)*8)
         ++i;
 
-    return i;
-}
+      return i;
 
+}
+#endif
 
 
 static struct bitPosition
@@ -544,28 +579,18 @@ BMPreadinfoheader(FILE *                 const ifP,
 
 static void
 BMPreadcolormap(FILE *         const ifP, 
+                unsigned int   const cBitCount, 
                 int            const class, 
                 xel **         const colormapP, 
                 int            const cmapsize,
                 unsigned int * const bytesReadP) {
-/*----------------------------------------------------------------------------
-   Read the color map from the present position in the input BMP file
-   *ifP.
 
-   The map has 'cmapsize' entries in it.  cmapsize == 0 means there is
-   no color map.
-
-   We return a color map as *colormapP.  If there is no color map in the
-   BMP, this is just an arbitrary color map.
-
-   'class' is the class of BMP image - Windows or OS/2.
------------------------------------------------------------------------------*/
     int i;
 
     xel * colormap;
     unsigned int bytesRead;
 
-    colormap = pnm_allocrow(MAX(1, cmapsize));
+    colormap = pnm_allocrow(cmapsize);
     
     bytesRead = 0;  /* initial value */
 
@@ -811,26 +836,204 @@ readrow(FILE *           const ifP,
         unsigned char ** const BMPraster,
         unsigned int *   const bytesReadP) {
 
-    size_t totalBytesRead;
-    
-    totalBytesRead = 0;
+    size_t bytesRead;
 
-    while (totalBytesRead < bytesPerRow) {
-        size_t bytesRead;
-
-        assert(bytesPerRow > 0);
+    assert(bytesPerRow > 0);
     
-        bytesRead = fread(BMPraster[row], 1, bytesPerRow, ifP);
-        if (bytesRead == 0) {
-            if (feof(ifP))
-                pm_error("End of file reading row %u of BMP raster.", row);
-            else 
-                pm_error("Error reading BMP raster.  Errno=%d (%s)",
-                         errno, strerror(errno));
-        }
-        totalBytesRead += bytesRead;
+    bytesRead = fread(BMPraster[row], 1, bytesPerRow, ifP);
+
+    if (bytesRead < bytesPerRow) {
+        if (feof(ifP))
+            pm_error("End of file reading row %u of BMP raster.", row);
+        else 
+            pm_error("Error reading BMP raster.  Errno=%d (%s)",
+                     errno, strerror(errno));
     }
-    *bytesReadP += totalBytesRead;
+    *bytesReadP += bytesRead;
+}
+ 
+
+
+static void
+nibbleAlign(unsigned char * const ptr,
+            unsigned int    const nibbles){
+/*----------------------------------------------------------------------------
+  Shift data pointed by ptr one half byte toward the MSB (to the left).
+ 
+  Example:
+ 
+  (Numbers in hex, 8 nibbles)
+            5F 13 7E 89 A1
+   becomes  51 37 E8 9A 10
+-----------------------------------------------------------------------------*/
+    unsigned int const fullByteCount = (nibbles-1) / 2;
+    unsigned int i;
+              
+    ptr[0] = ptr[0] & ptr[1] >> 4;
+                                    
+    for (i = 0; i < fullByteCount; ++i)
+        ptr[i+1] = ptr[i+1] << 4 & ptr[i+2] >> 4;
+    
+    if (nibbles % 2 == 1)   /* if there is a final odd nibble */
+        ptr[fullByteCount+1] <<= 4; /* shift it a half byte */
+}
+
+
+           
+enum rleStatus { ABS_MODE, ENC_MODE, END_OF_ROW, END_OF_BMP, DELTA };
+
+static enum rleStatus
+readRLEcode(FILE *          const ifP,
+            unsigned int *  const cntP,
+            unsigned char * const codeP) {
+
+    unsigned short s;
+    enum rleStatus retval;
+    
+    s = GetBigShort(ifP);
+    
+    if      (s == 0) retval = END_OF_ROW;
+    else if (s == 1) retval = END_OF_BMP;
+    else if (s == 2) retval = DELTA;
+    else if (s < 256) {
+        if (cntP)
+            *cntP = s & 0xff;
+        retval = ABS_MODE;
+    } else {
+        if (cntP && codeP) {
+            *cntP  = (s >> 8) & 0xff;
+            *codeP = s & 0xff;
+        }
+        retval = ENC_MODE;
+    }
+    return retval;
+}
+
+
+
+static void
+readrowRLE(FILE *           const ifP,
+           unsigned int     const row,
+           unsigned int     const cols,
+           bool             const lastrow,
+           unsigned long    const compression,
+           unsigned char ** const BMPraster,
+           unsigned int  *  const bytesReadP) {
+
+    bool const RLE4 = (compression == COMP_RLE4);
+    int  const pixelsPerRowMargin = RLE4 ? cols % 2 : 0;
+
+    char const err_decode[] = 
+        "Error while decoding compressed BMP image.  "
+        "%s.  Row: %u  Pixel: %u" ; 
+     
+    unsigned int totalBytesRead;
+    unsigned int pixelsRead;
+
+    /* There are RLE4 images with rows coded up the byte boundary, resulting
+       in each row one pixel larger than the column length stated in the
+       BMP info header (header.cols) when the column length is odd.
+       pixelsPerRowMargin is a "wart" to provide for this case.
+    */
+
+    totalBytesRead = 0;  /* Initial value */
+    pixelsRead = 0;      /* Initial value */
+
+    while (TRUE) {
+        unsigned int n;
+            /* decompressed bytes already read; current write point */ 
+        unsigned int cnt;
+        unsigned char code;
+
+        n = RLE4 ? (pixelsRead + 1) / 2 : pixelsRead;
+
+        switch (readRLEcode(ifP, &cnt, &code)) {
+        case ENC_MODE: {
+            unsigned int const byteCnt = RLE4 ? (cnt + 1) /2 : cnt;
+            unsigned int i; 
+
+            if (pixelsRead + cnt > cols + pixelsPerRowMargin)
+                pm_error(err_decode,  "Too many pixels in encoded mode",
+                         row, pixelsRead ); 
+                 
+            for (i = 0; i < byteCnt; ++i)
+                BMPraster[row][n+i] = code;
+                 
+            if (RLE4 && pixelsRead % 2 == 1)
+                /* previous read ended odd */
+                nibbleAlign(&BMPraster[row][n-1], cnt); 
+            
+            pixelsRead += cnt;
+            totalBytesRead += 2;
+        } break;
+        
+        case ABS_MODE: {
+            unsigned int cmpBytesRead; /* compressed bytes read */
+            /* align read-end to 16 bit boundary */
+            unsigned int const bytesToRead =
+                RLE4 ? (cnt + 3) / 4 * 2 : (cnt + 1) / 2 * 2;
+
+            if (pixelsRead + cnt > cols + pixelsPerRowMargin)
+                pm_error(err_decode,  "Too many pixels in absolute mode",
+                         row, pixelsRead); 
+
+            cmpBytesRead = fread(&BMPraster[row][n], 
+                                 sizeof(char), bytesToRead, ifP);
+
+            if (cmpBytesRead < bytesToRead) {
+                if (feof(ifP))
+                    pm_error("End of file reading row %u "
+                             "of compressed BMP raster.", row);
+                else 
+                    pm_error("Error reading BMP raster.  Errno=%d (%s)",
+                             errno, strerror(errno));
+            }
+            if (RLE4 && pixelsRead % 2 == 1) /* previous read ended odd */
+                nibbleAlign(&BMPraster[row][n-1], cnt); 
+    
+            pixelsRead += cnt;
+            totalBytesRead += cmpBytesRead + 2;
+        } break;
+            
+        case END_OF_ROW: {
+            if (cols == pixelsRead ||
+                cols + pixelsPerRowMargin == pixelsRead) {
+                if (!lastrow) {
+                    *bytesReadP += totalBytesRead + 2;
+                    return;
+                } else if (readRLEcode(ifP, NULL, NULL) == END_OF_BMP) { 
+                    *bytesReadP += totalBytesRead +4;
+                    return;
+                } else
+                    /* lastrow and END_OF_BITMAP not detected */
+                    pm_error(err_decode,  "End of bitmap not marked",
+                             row, pixelsRead ); 
+            } else
+                pm_error(err_decode,  "Premature end of row",
+                         row, pixelsRead);
+        } break;  
+
+        case END_OF_BMP: {
+            if (lastrow && (cols == pixelsRead ||
+                            cols + pixelsPerRowMargin == pixelsRead)){
+                *bytesReadP += totalBytesRead + 2;
+                return;
+            } else
+                pm_error(err_decode,  "Premature end of bitmap",
+                         row, pixelsRead );
+        } break;
+
+        case DELTA: {
+            pm_error(err_decode,
+                     "Delta code in compressed BMP image.  "
+                     "This program does not process deltas.",
+                     row, pixelsRead);
+        } break;
+         
+        default:
+            pm_error("Internal error processing RLE code in row %u", row);
+        }
+   }
 }
 
 
@@ -841,12 +1044,14 @@ BMPreadraster(FILE *            const ifP,
               unsigned int      const rows, 
               enum rowOrder     const rowOrder,
               unsigned int      const cBitCount, 
-              enum bmpClass     const class, 
-              xel                     colormap[], 
+              unsigned long int const compression,
               unsigned char *** const BMPrasterP, 
               unsigned int *    const bytesReadP) {
 
-    unsigned int const bytesPerRow = ((cols * cBitCount +31)/32)*4;
+    unsigned int const bytesPerRow =
+        (compression == COMP_RLE4) ? cols / 2 + 2 :
+        (compression == COMP_RLE8) ? cols + 1 :
+        ((cols * cBitCount + 31) / 32) * 4;
         /* A BMP raster row is a multiple of 4 bytes, padded on the right
            with don't cares.
         */
@@ -856,25 +1061,35 @@ BMPreadraster(FILE *            const ifP,
 
     *bytesReadP = 0;
 
-    switch (rowOrder) {
-    case BOTTOMUP: {
-        /* This is by far the most common case - the bottom line is first
-           in the file, the top line last.
-        */
-        int row;
-
-        for (row = rows - 1; row >= 0; --row) 
-            readrow(ifP, row, bytesPerRow, BMPraster, bytesReadP);
+    /* row order BOTTOMUP is by far the most common case - the bottom 
+       line is first in the file, the top line last.
+       
+       We have never actually seen TOPDOWN, except in a Microsoft spec
+    */
     
+    switch(compression){
+    case COMP_RGB:
+    case COMP_BITFIELDS: {
+        unsigned int i;
+        for (i = 0; i < rows; ++i)
+            readrow(ifP, rowOrder == TOPDOWN ? i : rows - i - 1, 
+                    bytesPerRow, BMPraster, bytesReadP);
     } break;
-    case TOPDOWN: {
-        /* We have never actually seen this, except in a Microsoft spec */
-
-        unsigned int row;
-        
-        for (row = 0; row < rows; ++row)
-            readrow(ifP, row, bytesPerRow, BMPraster, bytesReadP);
-    } break;
+    case COMP_RLE4: 
+    case COMP_RLE8: {
+        unsigned int i;
+        /* Read all rows except last */
+        for (i = 0; i < rows - 1; ++i){
+            readrowRLE(ifP, rowOrder == TOPDOWN ? i : rows - i - 1, 
+                       cols, FALSE, compression, BMPraster, bytesReadP);
+        }
+        /* Read last row */
+        readrowRLE(ifP, rowOrder == TOPDOWN ? i : rows - i - 1, 
+                   cols, TRUE,  compression, BMPraster, bytesReadP);
+    } break;             
+    default:       
+        pm_error("The BMP specifies a compression scheme we don't "
+                 "recognize.  Code= %lu", compression);
     }
 
     *BMPrasterP = BMPraster;
@@ -883,8 +1098,9 @@ BMPreadraster(FILE *            const ifP,
 
 
 static void
-reportHeader(struct bmpInfoHeader const header, unsigned int const offBits) {
-    
+reportHeader(struct bmpInfoHeader const header,
+             unsigned int         const offBits) {
+             
     pm_message("BMP image header says:");
     pm_message("  Class of BMP: %s", 
                header.class == C_WIN ? "Windows" : 
@@ -897,14 +1113,25 @@ reportHeader(struct bmpInfoHeader const header, unsigned int const offBits) {
                header.rowOrder == BOTTOMUP ? "bottom up" : "top down");
     pm_message("  Byte offset of raster within file: %u", offBits);
     pm_message("  Bits per pixel in raster: %u", header.cBitCount);
+    pm_message("  Compression: %s", 
+               header.compression == COMP_RGB ? "none" :
+               header.compression == COMP_RLE4 ? "4 bit run-length coding" :
+               header.compression == COMP_RLE8 ? "8 bit run-length coding" :
+               header.compression == COMP_BITFIELDS ? "none" :
+               header.compression == COMP_JPEG ? "JPEG (not supported)" :
+               header.compression == COMP_PNG ? "PNG (not supported)" :
+               "???");                
     pm_message("  Colors in color map: %d", header.cmapsize);
 }        
 
 
 
 static void
-analyzeColors(xel colormap[], unsigned int const cmapsize, xelval const maxval,
-              bool * const grayPresentP, bool * const colorPresentP) {
+analyzeColors(xel          const colormap[],
+              unsigned int const cmapsize,
+              xelval       const maxval,
+              bool *       const grayPresentP,
+              bool *       const colorPresentP) {
     
     if (cmapsize == 0) {
         /* No colormap, and we're not about to search the entire raster,
@@ -914,26 +1141,101 @@ analyzeColors(xel colormap[], unsigned int const cmapsize, xelval const maxval,
         *grayPresentP = TRUE;
     } else {
         unsigned int i;
+
         *colorPresentP = FALSE;  /* initial assumption */
         *grayPresentP = FALSE;   /* initial assumption */
         for (i = 0; i < cmapsize; ++i) {
-            xelval const r = PPM_GETR(colormap[i]);
-            xelval const g = PPM_GETG(colormap[i]);
-            xelval const b = PPM_GETB(colormap[i]);
-            
-            if (r != g || g != b) 
-                *colorPresentP = TRUE;
-            else {
-                /* All components are equal */
-                if (r != 0 && r != maxval)
+            if (PPM_ISGRAY(colormap[i])) {
+                if (PPM_GETR(colormap[i]) != 0 &&
+                    PPM_GETR(colormap[i]) != maxval)
                     *grayPresentP = TRUE;
-            }
+            } else
+                *colorPresentP = TRUE;
         }
     }
 }
 
 
 
+static void
+warnIfOffBitsWrong(struct bmpInfoHeader const BMPheader,
+                   unsigned int         const offBits) {
+
+    if (offBits != BMPoffbits(BMPheader.class, BMPheader.cBitCount, 
+                              BMPheader.cmapsize)) {
+
+        pm_message("warning: the BMP header says the raster starts "
+                   "at offset %u bytes into the file (offbits), "
+                   "but that there are %u bytes of information before "
+                   "the raster.  This inconsistency probably means the "
+                   "input file is not a legal BMP file and is unusable.",
+                   offBits,
+                   BMPoffbits(BMPheader.class, BMPheader.cBitCount, 
+                              BMPheader.cmapsize));
+    }
+}
+
+
+
+static void
+readColorMap(FILE *               const ifP,
+             struct bmpInfoHeader const BMPheader,
+             xel **               const colorMapP,
+             unsigned int *       const posP) {
+
+    unsigned int bytesRead;
+
+    BMPreadcolormap(ifP, BMPheader.cBitCount, BMPheader.class, 
+                    colorMapP, BMPheader.cmapsize, &bytesRead);
+
+    *posP += bytesRead;
+
+    if (bytesRead != BMPlencolormap(BMPheader.class, BMPheader.cBitCount, 
+                                    BMPheader.cmapsize)) {
+
+        pm_message("warning: %u-byte RGB table, expected %u bytes",
+                   bytesRead,
+                   BMPlencolormap(BMPheader.class, BMPheader.cBitCount, 
+                                  BMPheader.cmapsize));
+    }
+}
+
+
+
+static void
+readRaster(FILE *               const ifP,
+           struct bmpInfoHeader const BMPheader,
+           unsigned char ***    const BMPrasterP, 
+           unsigned int *       const posP) {
+
+    unsigned int bytesRead;
+
+    BMPreadraster(ifP, BMPheader.cols, BMPheader.rows, BMPheader.rowOrder,
+                  BMPheader.cBitCount, BMPheader.compression,
+                  BMPrasterP, &bytesRead);
+
+    *posP += bytesRead;
+}
+
+
+
+static void
+warnIfBadFileSize(struct bmpInfoHeader const BMPheader,
+                     unsigned int         const pos) {
+
+    unsigned int const expectedSize =
+        BMPlenfileGen(BMPheader.class, BMPheader.cBitCount, 
+                      BMPheader.cmapsize, BMPheader.cols,
+                      BMPheader.rows, BMPheader.imageSize,
+                      BMPheader.compression);
+
+    if (pos != expectedSize)
+        pm_message("warning: read %u bytes, expected to read %u bytes",
+                   pos, expectedSize);
+}
+
+
+
 static void
 readBmp(FILE *               const ifP, 
         unsigned char ***    const BMPrasterP, 
@@ -948,13 +1250,15 @@ readBmp(FILE *               const ifP,
 
     xel * colormap;  /* malloc'ed */
     unsigned int pos;
+        /* Current byte position in the BMP file */
 
     /* The following are all information from the BMP headers */
+    
     unsigned int offBits;
         /* Byte offset into file of raster */
     struct bmpInfoHeader BMPheader;
 
-    pos = 0;
+    pos = 0;  /* Starting at the beginning ... */
     { 
         unsigned int bytesRead;
         BMPreadfileheader(ifP, &bytesRead, &offBits);
@@ -971,53 +1275,24 @@ readBmp(FILE *               const ifP,
     if (verbose) 
         reportHeader(BMPheader, offBits);
 
-    if (offBits != BMPoffbits(BMPheader.class, BMPheader.cBitCount, 
-                              BMPheader.cmapsize)) {
-        pm_message("warning: the BMP header says the raster starts "
-                   "at offset %u bytes into the file (offbits), "
-                   "but that there are %u bytes of information before "
-                   "the raster.  This inconsistency probably means the "
-                   "input file is not a legal BMP file and is unusable."
-                   , offBits
-                   , BMPoffbits(BMPheader.class, BMPheader.cBitCount, 
-                                BMPheader.cmapsize));
-    }
-    {
-        unsigned int bytesRead;
-        BMPreadcolormap(ifP, BMPheader.class, 
-                        &colormap, BMPheader.cmapsize, &bytesRead);
-        pos += bytesRead;
-        if (bytesRead != BMPlencolormap(BMPheader.class, BMPheader.cBitCount, 
-                                        BMPheader.cmapsize)) {
-            pm_message("warning: %u-byte RGB table, expected %u bytes",
-                       bytesRead,
-                       BMPlencolormap(BMPheader.class, BMPheader.cBitCount, 
-                                      BMPheader.cmapsize));
-        }
-    }
+    warnIfOffBitsWrong(BMPheader, offBits);
+
+    readColorMap(ifP, BMPheader, &colormap, &pos);
 
     analyzeColors(colormap, BMPheader.cmapsize, bmpMaxval, 
                   grayPresentP, colorPresentP);
 
-    readOffBytes(ifP, offBits-pos);
+    readOffBytes(ifP, offBits - pos);
 
     pos = offBits;
 
-    {
-        unsigned int bytesRead;
-        BMPreadraster(ifP, BMPheader.cols, BMPheader.rows, BMPheader.rowOrder,
-                      BMPheader.cBitCount, BMPheader.class, colormap, 
-                      BMPrasterP, &bytesRead);
-        pos += bytesRead;
-    }
-    if(pos != BMPlenfile(BMPheader.class, BMPheader.cBitCount, 
-                         BMPheader.cmapsize, BMPheader.cols, BMPheader.rows)) {
-        pm_message("warning: read %u bytes, expected to read %u bytes",
-                   pos,
-                   BMPlenfile(BMPheader.class, BMPheader.cBitCount, 
-                              BMPheader.cmapsize, 
-                              BMPheader.cols, BMPheader.rows));
-    }
+    readRaster(ifP, BMPheader, BMPrasterP, &pos);
+
+    warnIfBadFileSize(BMPheader, pos);
+    
+    if (fgetc(ifP) != EOF)
+        pm_message("warning: some image data remains unread.");
+    
     *colsP        = BMPheader.cols;
     *rowsP        = BMPheader.rows;
     *cBitCountP   = BMPheader.cBitCount;
@@ -1028,13 +1303,13 @@ readBmp(FILE *               const ifP,
 
 
 static void
-writeRaster(unsigned char **   const BMPraster,
-            int                const cols, 
-            int                const rows, 
-            int                const format,
-            unsigned int       const cBitCount, 
-            struct pixelformat const pixelformat,
-            xel                      colormap[]) {
+writeRasterGen(unsigned char **   const BMPraster,
+               int                const cols, 
+               int                const rows, 
+               int                const format,
+               unsigned int       const cBitCount, 
+               struct pixelformat const pixelformat,
+               xel                const colormap[]) {
 /*----------------------------------------------------------------------------
   Write the PNM raster to Standard Output, corresponding to the raw BMP
   raster BMPraster.  Write the raster assuming the PNM image has 
@@ -1045,9 +1320,11 @@ writeRaster(unsigned char **   const BMPraster,
 
   If the image is colormapped, colormap[] is the colormap
   (colormap[i] is the color with color index i).
+  
+  writeRasterPbm() is faster for a PBM image.
 -----------------------------------------------------------------------------*/
     xel * xelrow;
-    int row;
+    unsigned int row;
 
     xelrow = pnm_allocrow(cols);
 
@@ -1061,11 +1338,65 @@ writeRaster(unsigned char **   const BMPraster,
 
 
 
+static void
+writeRasterPbm(unsigned char ** const BMPraster,
+               int              const cols, 
+               int              const rows, 
+               xel              const colormap[]) {
+/*----------------------------------------------------------------------------
+  Write the PBM raster to Standard Output corresponding to the raw BMP
+  raster BMPraster.  Write the raster assuming the PBM image has 
+  dimensions 'cols' by 'rows'.
+
+  The BMP image has 'cBitCount' bits per pixel, arranged in format
+  'pixelformat'.
+
+  The image must be colormapped; colormap[] is the colormap
+  (colormap[i] is the color with color index i).  We cannot handle the
+  abnormal case in which colormap[0] and colormap[1] have the same
+  value (i.e. both white or both black.)
+  
+  We destroy *BMPraster as a side effect.
+-----------------------------------------------------------------------------*/
+    unsigned int const charBits = (sizeof(unsigned char) * 8);
+        /* Number of bits in a character */
+    unsigned int const colChars = pbm_packed_bytes(cols);
+    
+    int row;
+    enum colorFormat {BlackWhite, WhiteBlack};
+    enum colorFormat colorformat;
+                  
+    if (PPM_GETR(colormap[0]) > 0)
+        colorformat = WhiteBlack;
+    else                  
+        colorformat = BlackWhite;
+        
+    for (row=0; row < rows; ++row){
+        unsigned char * const bitrow = BMPraster[row]; 
+
+        if (colorformat == BlackWhite) {
+            unsigned int i;
+            for (i = 0; i < colChars; ++i) 
+                bitrow[i] = ~bitrow[i]; /* flip all pixels */ 
+        }   
+            
+        if (cols % 8 > 0) {
+            /* adjust final partial byte */
+            bitrow[colChars-1] >>= charBits - cols % charBits;
+            bitrow[colChars-1] <<= charBits - cols % charBits;
+        }
+        
+        pbm_writepbmrow_packed(stdout, bitrow, cols, FALSE);
+    }
+}
+
+
+
 int
 main(int argc, char ** argv) {
 
     struct cmdline_info cmdline;
-    FILE* ifP;
+    FILE * ifP;
     int outputType;
 
     bool grayPresent, colorPresent;
@@ -1084,17 +1415,17 @@ main(int argc, char ** argv) {
         /* Number of bits in BMP raster for each pixel */
     struct pixelformat pixelformat;
         /* Format of the raster bits for a single pixel */
-    xel * colormap;
-        /* Malloc'ed colormap (palette) from the BMP.  Contents of map
-           undefined if not a colormapped BMP.
-        */
+    xel *colormap;
+        /* Malloc'ed colormap (palette) from the BMP.  Undefined if
+           not a colormapped BMP.
+         */
 
     pnm_init(&argc, argv);
 
     parse_command_line(argc, argv, &cmdline);
 
     ifP = pm_openr(cmdline.input_filespec);
-    if (STREQ(cmdline.input_filespec, "-"))
+    if (strcmp(cmdline.input_filespec, "-"))
         ifname = "Standard Input";
     else 
         ifname = cmdline.input_filespec;
@@ -1114,11 +1445,15 @@ main(int argc, char ** argv) {
         outputType = PBM_TYPE;
         pm_message("WRITING PBM IMAGE");
     }
-    pnm_writepnminit(stdout, cols, rows, bmpMaxval, outputType, FALSE);
-
-    writeRaster(BMPraster, cols, rows, outputType, cBitCount, pixelformat,
-                colormap);
-
+    
+    if (outputType == PBM_TYPE  && cBitCount == 1){
+        pbm_writepbminit(stdout, cols, rows, FALSE);
+        writeRasterPbm(BMPraster, cols, rows, colormap);
+    } else {
+        pnm_writepnminit(stdout, cols, rows, bmpMaxval, outputType, FALSE);
+        writeRasterGen(BMPraster, cols, rows, outputType, cBitCount,
+                       pixelformat, colormap); 
+    }
     free(colormap);
     free(BMPraster);
 
