@@ -6,8 +6,20 @@
    information on copyright and licensing.
  */
 
+#define _BSD_SOURCE 1   /* Make sure string.h contains strcasecmp() */
+
+/* In Solaris 9, there is a bug in sys/resource.h, which gets included
+   via netinet/in.h: It defines struct rusage with members of type
+   timeval, but does not define type timeval when _XOPEN_SOURCE is
+   true, as it is here.  The fact that time.h doesn't define timeval
+   when _XOPEN_SOURCE is true is a second bug (_XOPEN_SOURCE is
+   supposed to add features, not subtract them).  We work around both
+   by including time.h without _XOPEN_SOURCE.
+*/
+#include <time.h>
+
 #define _XOPEN_SOURCE  /* Make sure unistd.h contains swab() */
-#define _BSD_SOURCE    /* Make sure string.h containst strcasecmp() */
+
 #include <ctype.h>
 #include <unistd.h>
 #include <errno.h>
@@ -19,7 +31,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
 
 #ifdef __CYGWIN__
 #include <io.h>
@@ -53,8 +64,7 @@
 #else
    /* We define INT64 to something that lets the code compile, but we
       should not execute any INT64 code, because it will get the wrong
-      result.
-   */
+      result.  */
    typedef int INT64;
    static bool const have64BitArithmetic = false;
 #endif
