@@ -41,7 +41,18 @@
 #define LOCALCOLORMAP  0x80
 #define BitSet(byte, bit)      (((byte) & (bit)) == (bit))
 
-#define        ReadOK(file,buffer,len) (fread(buffer, len, 1, file) != 0)
+static __inline__ bool
+ReadOK(FILE *          const fileP,
+       unsigned char * const buffer,
+       size_t          const len) {
+
+    size_t bytesRead;
+
+    bytesRead = fread(buffer, len, 1, fileP);
+
+    return (bytesRead != 0);
+}
+
 
 #define LM_to_uint(a,b)                        (((b)<<8)|(a))
 
@@ -720,6 +731,12 @@ lzwInit(struct decompressor * const decompP,
     decompP->max_dataVal = (1 << init_codeSize) - 1;
     decompP->clear_code = decompP->max_dataVal + 1;
     decompP->end_code = decompP->max_dataVal + 2;
+
+    if (verbose)
+        pm_message("Initial code size is %u bits; clear code = 0x%x, "
+                   "end code = 0x%x",
+                   decompP->init_codeSize, 
+                   decompP->clear_code, decompP->end_code);
     
     /* The entries in the translation table for true data codes are
        constant throughout the stream.  We set them now and they never
@@ -738,7 +755,7 @@ lzwInit(struct decompressor * const decompP,
     
     decompP->fresh = TRUE;
     
-    initStack(&decompP->stack, maxnum_lzwCode*2);
+    initStack(&decompP->stack, maxnum_lzwCode * 2);
 }
 
 
@@ -834,7 +851,7 @@ lzwReadByte(struct decompressor * const decompP) {
   Return the next data element of the decompressed image.  In the context
   of a GIF, a data element is the color table index of one pixel.
 
-  We read and return the next byte of the decompressed image:
+  We read and return the next byte of the decompressed image, or:
 
     Return -1 if we hit EOF prematurely (i.e. before an "end" code.  We
     forgive the case that the "end" code is followed by EOF instead of
@@ -848,7 +865,7 @@ lzwReadByte(struct decompressor * const decompP) {
 -----------------------------------------------------------------------------*/
     int retval;
 
-    if (!stackIsEmpty(&decompP->stack)) 
+    if (!stackIsEmpty(&decompP->stack))
         retval = popStack(&decompP->stack);
     else if (decompP->fresh) {
         decompP->fresh = FALSE;
@@ -861,7 +878,12 @@ lzwReadByte(struct decompressor * const decompP) {
                 getCode(decompP->ifP, decompP->codeSize, FALSE);
             decompP->prevcode = decompP->firstcode;
         } while (decompP->firstcode == decompP->clear_code);
-        retval = decompP->firstcode;
+        if (decompP->firstcode == decompP->end_code) {
+            if (!zeroDataBlock)
+                readThroughEod(decompP->ifP);
+            retval = -2;
+        } else
+            retval = decompP->firstcode;
     } else {
         int code;
         code = getCode(decompP->ifP, decompP->codeSize, FALSE);
@@ -871,18 +893,20 @@ lzwReadByte(struct decompressor * const decompP) {
             assert(code >= 0);  /* -1 is only possible error return */
             if (code == decompP->clear_code) {
                 resetDecompressor(decompP);
-                retval = lzwReadByte( decompP );
-            } else if (code == decompP->end_code) {
-                if (!zeroDataBlock)
-                    readThroughEod(decompP->ifP);
-                retval = -2;
+                retval = lzwReadByte(decompP);
             } else {
-                bool error;
-                expandCodeOntoStack(decompP, code, &error);
-                if (error)
-                    retval = -3;
-                else
-                    retval = popStack(&decompP->stack);
+                if (code == decompP->end_code) {
+                    if (!zeroDataBlock)
+                        readThroughEod(decompP->ifP);
+                    retval = -2;
+                } else {
+                    bool error;
+                    expandCodeOntoStack(decompP, code, &error);
+                    if (error)
+                        retval = -3;
+                    else
+                        retval = popStack(&decompP->stack);
+                }
             }
         }
     }
@@ -891,6 +915,109 @@ lzwReadByte(struct decompressor * const decompP) {
 
 
 
+enum pass {MULT8PLUS0, MULT8PLUS4, MULT4PLUS2, MULT2PLUS1};
+
+static void
+bumpRowInterlace(unsigned int * const rowP,
+                 unsigned int   const rows,
+                 enum pass *    const passP) {
+/*----------------------------------------------------------------------------
+   Move *pixelCursorP to the next row in the interlace pattern.
+-----------------------------------------------------------------------------*/
+    /* There are 4 passes:
+       MULT8PLUS0: Rows 8, 16, 24, 32, etc.
+       MULT8PLUS4: Rows 4, 12, 20, 28, etc.
+       MULT4PLUS2: Rows 2, 6, 10, 14, etc.
+       MULT2PLUS1: Rows 1, 3, 5, 7, 9, etc.
+    */
+    
+    switch (*passP) {
+    case MULT8PLUS0:
+        *rowP += 8;
+        break;
+    case MULT8PLUS4:
+        *rowP += 8;
+        break;
+    case MULT4PLUS2:
+        *rowP += 4;
+        break;
+    case MULT2PLUS1:
+        *rowP += 2;
+        break;
+    }
+    /* Set the proper pass for the next read.  Note that if there are
+       more than 4 rows, the sequence of passes is sequential, but
+       when there are fewer than 4, we may skip e.g. from MULT8PLUS0
+       to MULT4PLUS2.
+    */
+    while (*rowP >= rows && *passP != MULT2PLUS1) {
+        switch (*passP) {
+        case MULT8PLUS0:
+            *passP = MULT8PLUS4;
+            *rowP = 4;
+            break;
+        case MULT8PLUS4:
+            *passP = MULT4PLUS2;
+            *rowP = 2;
+            break;
+        case MULT4PLUS2:
+            *passP = MULT2PLUS1;
+            *rowP = 1;
+            break;
+        case MULT2PLUS1:
+            /* We've read the entire image */
+            break;
+        }
+    }
+}
+
+
+
+struct pnmBuffer {
+    xel ** xels;
+    unsigned int col;
+    unsigned int row;
+};
+
+static void
+addPixelToRaster(unsigned int       const cmapIndex,
+                 struct pnmBuffer * const pnmBufferP,
+                 unsigned int       const cols,
+                 unsigned int       const rows,
+                 gifColorMap              cmap, 
+                 unsigned int       const cmapSize,
+                 bool               const interlace,
+                 int                const transparentIndex,
+                 bit **             const alphabits,
+                 enum pass *        const passP) {
+
+    if (cmapIndex >= cmapSize)
+        pm_error("Invalid color index %u in an image that has only "
+                 "%u colors in the color map.", cmapIndex, cmapSize);
+    
+    assert(cmapIndex < MAXCOLORMAPSIZE);
+    
+    PPM_ASSIGN(pnmBufferP->xels[pnmBufferP->row][pnmBufferP->col], 
+               cmap[CM_RED][cmapIndex],
+               cmap[CM_GRN][cmapIndex],
+               cmap[CM_BLU][cmapIndex]);
+    
+    if (alphabits) 
+        alphabits[pnmBufferP->row][pnmBufferP->col] =
+            (cmapIndex == transparentIndex) ? PBM_BLACK : PBM_WHITE;
+    
+    ++pnmBufferP->col;
+    if (pnmBufferP->col == cols) {
+        pnmBufferP->col = 0;
+        if (interlace)
+            bumpRowInterlace(&pnmBufferP->row, rows, passP);
+        else
+            ++pnmBufferP->row;
+    }
+}
+
+
+
 static void
 readImageData(FILE *       const ifP, 
               xel **       const xels, 
@@ -903,19 +1030,19 @@ readImageData(FILE *       const ifP,
               bit **       const alphabits) {
 
     unsigned char lzwMinCodeSize;      
-    int v;
-    int xpos, ypos, pass;
+    enum pass pass;
     struct decompressor decomp;
+    struct pnmBuffer pnmBuffer;
+    bool gotMinCodeSize;
 
-    pass = 0;
-    xpos = 0;
-    ypos = 0;
+    pass = MULT8PLUS0;
 
-    
-    /*
-    **  Initialize the Compression routines
-    */
-    if (! ReadOK(ifP,&lzwMinCodeSize,1))
+    pnmBuffer.xels = xels;
+    pnmBuffer.col  = 0;
+    pnmBuffer.row  = 0;
+
+    gotMinCodeSize =  ReadOK(ifP, &lzwMinCodeSize, 1);
+    if (!gotMinCodeSize)
         pm_error("GIF stream ends (or read error) "
                  "right after an image separator; no "
                  "image data follows.");
@@ -927,61 +1054,30 @@ readImageData(FILE *       const ifP,
 
     lzwInit(&decomp, ifP, lzwMinCodeSize);
 
-    while ((v = lzwReadByte(&decomp)) >= 0 ) {
-        if (v >= cmapSize)
-            pm_error("Invalid color index %u in an image that has only "
-                     "%u colors in the color map.", v, cmapSize);
+    while (pnmBuffer.row < rows) {
+        int const rc = lzwReadByte(&decomp);
 
-        assert(v < MAXCOLORMAPSIZE);
-
-        PPM_ASSIGN(xels[ypos][xpos], 
-                   cmap[CM_RED][v], cmap[CM_GRN][v], cmap[CM_BLU][v]);
-
-        if (alphabits) 
-            alphabits[ypos][xpos] = (v == transparentIndex) ? 
-                PBM_BLACK : PBM_WHITE;
-
-        ++xpos;
-        if (xpos == cols) {
-            xpos = 0;
-            if (interlace) {
-                switch (pass) {
-                case 0:
-                case 1:
-                    ypos += 8; break;
-                case 2:
-                    ypos += 4; break;
-                case 3:
-                    ypos += 2; break;
-                }
-                
-                if (ypos >= rows) {
-                    ++pass;
-                    switch (pass) {
-                    case 1:
-                        ypos = 4; break;
-                    case 2:
-                        ypos = 2; break;
-                    case 3:
-                        ypos = 1; break;
-                    default:
-                        goto fini;
-                    }
-                }
-            } else
-                ++ypos;
-        }
-        if (ypos >= rows)
+        switch (rc) {
+        case -3:
+            pm_error("Error in GIF input stream");
             break;
+        case -2:
+            pm_error("Error in GIF image: Not enough raster data to fill "
+                     "%u x %u dimensions.  Ran out of raster data in "
+                     "row %u", cols, rows, pnmBuffer.row);
+            break;
+        case -1:
+            pm_error("Premature end of file; no proper GIF closing");
+            break;
+        default:
+            addPixelToRaster(rc, &pnmBuffer, cols, rows, cmap, cmapSize,
+                             interlace, transparentIndex, alphabits, &pass);
+        }
     }
-fini:
-    if (v == -3)
-        pm_error("Error in GIF input stream");
-    else {
-        if (lzwReadByte(&decomp) >= 0)
-            pm_message("Extraneous data at end of image.  "
-                       "Skipped to end of image");
-    }
+    if (lzwReadByte(&decomp) >= 0)
+        pm_message("Extraneous data at end of image.  "
+                   "Skipped to end of image");
+
     lzwTerm(&decomp);
 }
 
@@ -1304,7 +1400,8 @@ convertImages(FILE * const ifP,
                          requestedImageSeq+1,
                          imageSeq, imageSeq>1?"s":"" );
         } else {
-            pm_message("Reading Image Sequence %d", imageSeq);
+            if (verbose)
+                pm_message("Reading Image Sequence %d", imageSeq);
             convertImage(ifP, !allImages && (imageSeq != requestedImageSeq), 
                          imageout_file, alphafile, gifScreen, gif89);
         }
