@@ -32,6 +32,7 @@
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
+#include "nstring.h"
 
 #define EXIT_WARNING 2   /* Goes with EXIT_SUCCESS, EXIT_FAILURE in stdlib.h */
 
@@ -177,11 +178,10 @@ interpret_density(const char *        const densityString,
                 if (matched < 3) 
                     densityP->unit = DEN_UNSPECIFIED;
                 else {
-                    if (strcmp(unitName, "dpi") == 0 || 
-                        strcmp(unitName, "DPI") == 0)
+                    if (STREQ(unitName, "dpi") || STREQ(unitName, "DPI"))
                         densityP->unit = DEN_DOTS_PER_INCH;
-                    else if (strcmp(unitName, "dpcm") == 0 || 
-                        strcmp(unitName, "DPCM") == 0)
+                    else if (STREQ(unitName, "dpcm") ||
+                             STREQ(unitName, "DPCM"))
                         densityP->unit = DEN_DOTS_PER_CM;
                     else
                         pm_error("Unrecognized unit '%s' in the density value "
@@ -293,16 +293,16 @@ parseCommandLine(const int argc, char ** argv,
     else if (argc_parse - 1 == 1)
         cmdlineP->input_filespec = strdup(argv_parse[1]);
     else 
-        pm_error("Too many arguments.  The only argument accepted"
+        pm_error("Too many arguments.  The only argument accepted "
                  "is the input file specification.");
     if (dctval == NULL)
         cmdlineP->dct_method = JDCT_DEFAULT;
     else {
-        if (strcmp(dctval, "int") == 0)
+        if (STREQ(dctval, "int"))
             cmdlineP->dct_method = JDCT_ISLOW;
-        else if (strcmp(dctval, "fast") == 0)
+        else if (STREQ(dctval, "fast"))
             cmdlineP->dct_method = JDCT_IFAST;
-        else if (strcmp(dctval, "float") == 0)
+        else if (STREQ(dctval, "float"))
             cmdlineP->dct_method = JDCT_FLOAT;
         else
             pm_error("Invalid value for the --dct option: '%s'.", dctval);
@@ -318,9 +318,9 @@ parseCommandLine(const int argc, char ** argv,
         pm_error("Smoothing factor %d is greater than 100 (%%).",
                  cmdlineP->smoothing_factor);
 
-    if (strcmp(cmdlineP->input_filespec, "=") == 0 &&
+    if (STREQ(cmdlineP->input_filespec, "=") &&
         cmdlineP->exif_filespec && 
-        strcmp(cmdlineP->exif_filespec, "-") == 0)
+        STREQ(cmdlineP->exif_filespec, "-"))
 
         pm_error("Cannot have both input image and exif header be from "
                  "Standard Input.");
