@@ -13,6 +13,9 @@
 
   Bryan used ideas on processing G3 data from Haeberli's code, but did
   not use any of the code.
+
+  Others have modified the program since Bryan's initial work, each
+  contributing their work to the public domain.
 ===========================================================================*/
 
 /* There is some indication that a valid fax always has 1728 columns
@@ -29,6 +32,7 @@
 #include "nstring.h"
 #include "mallocvar.h"
 #include "g3.h"
+#include "bitreverse.h"
 
 #define MAXCOLS 10800
 #define MAXROWS 14400   /* this allows up to two pages of image */
@@ -40,8 +44,9 @@
 #define BHASHB 2695
 
 #define HASHSIZE 1021
-static tableentry * whash[HASHSIZE];
-static tableentry * bhash[HASHSIZE];
+
+static g3TableEntry * whash[HASHSIZE];
+static g3TableEntry * bhash[HASHSIZE];
 
 
 struct cmdlineInfo {
@@ -115,6 +120,8 @@ struct bitStream {
         */
 };
 
+
+
 static void
 readBit(struct bitStream * const bitStreamP,
         unsigned int *     const bitP,
@@ -131,20 +138,19 @@ readBit(struct bitStream * const bitStreamP,
         bitStreamP->shdata = getc(bitStreamP->fileP);
         if (bitStreamP->shdata == EOF)
             asprintfN(errorP, "EOF or error reading file");
-        else
-            bitStreamP->shbit = bitStreamP->reversebits ? 0x01 : 0x80;
+        else {
+            bitStreamP->shbit = 0x80;
+            if ( bitStreamP->reversebits )
+                bitStreamP->shdata = bitreverse[ bitStreamP->shdata ];
+            }
     }
 
-    if (bitStreamP->shdata & bitStreamP->shbit) {
+    if (bitStreamP->shdata & bitStreamP->shbit)
         *bitP = 1;
-    } else {
-        *bitP = 0;
-    }
-    
-    if (bitStreamP->reversebits)
-        bitStreamP->shbit <<= 1;
     else
-        bitStreamP->shbit >>= 1;
+        *bitP = 0;
+
+    bitStreamP->shbit >>= 1;
 }
 
 
@@ -207,17 +213,18 @@ skipToNextLine(struct bitStream * const bitStreamP) {
 
 
 static void
-addtohash(tableentry *       hash[], 
-          tableentry         table[], 
-          int          const n, 
+addtohash(g3TableEntry *     hash[], 
+          g3TableEntry       table[], 
+          unsigned int const n, 
           int          const a, 
           int          const b) {
-
+    
     unsigned int i;
 
     for (i = 0; i < n; ++i) {
-        tableentry * const teP = &table[i];
-        unsigned int const pos = ((teP->length+a)*(teP->code+b))%HASHSIZE;
+        g3TableEntry * const teP = &table[i*2];
+        unsigned int const pos =
+            ((teP->length + a) * (teP->code + b)) % HASHSIZE;
         if (hash[pos])
             pm_error("internal error: addtohash fatal hash collision");
         hash[pos] = teP;
@@ -226,17 +233,17 @@ addtohash(tableentry *       hash[],
 
 
 
-static tableentry*
-hashfind(tableentry *       hash[], 
+static g3TableEntry*
+hashfind(g3TableEntry *       hash[], 
          int          const length, 
          int          const code, 
          int          const a, 
          int          const b) {
 
     unsigned int pos;
-    tableentry* te;
+    g3TableEntry * te;
 
-    pos = ((length+a)*(code+b))%HASHSIZE;
+    pos = ((length + a) * (code + b)) % HASHSIZE;
     te = hash[pos];
     return ((te && te->length == length && te->code == code) ? te : 0);
 }
@@ -244,23 +251,20 @@ hashfind(tableentry *       hash[],
 
 
 static void
-buildHashes(tableentry * (*whashP)[HASHSIZE],
-            tableentry * (*bhashP)[HASHSIZE]) {
+buildHashes(g3TableEntry * (*whashP)[HASHSIZE],
+            g3TableEntry * (*bhashP)[HASHSIZE]) {
 
     unsigned int i;
 
     for (i = 0; i < HASHSIZE; ++i)
         (*whashP)[i] = (*bhashP)[i] = NULL;
 
-#define TABSIZE(tab) (sizeof(tab)/sizeof(struct tableentry))
+    addtohash(*whashP, &ttable[0], 64, WHASHA, WHASHB);
+    addtohash(*whashP, &mtable[2], 40, WHASHA, WHASHB);
 
-    addtohash((*whashP), twtable, TABSIZE(twtable), WHASHA, WHASHB);
-    addtohash((*whashP), mwtable, TABSIZE(mwtable), WHASHA, WHASHB);
-    addtohash((*whashP), extable, TABSIZE(extable), WHASHA, WHASHB);
+    addtohash(*bhashP, &ttable[1], 64, BHASHA, BHASHB);
+    addtohash(*bhashP, &mtable[3], 40, BHASHA, BHASHB);
 
-    addtohash((*bhashP), tbtable, TABSIZE(tbtable), BHASHA, BHASHB);
-    addtohash((*bhashP), mbtable, TABSIZE(mbtable), BHASHA, BHASHB);
-    addtohash((*bhashP), extable, TABSIZE(extable), BHASHA, BHASHB);
 }
 
 
@@ -276,12 +280,12 @@ makeRowWhite(bit *        const bitrow,
 
 
 
-static tableentry *
+static g3TableEntry *
 g3code(unsigned int const curcode,
        unsigned int const curlen,
        bit          const color) {
 
-    tableentry * retval;
+    g3TableEntry * retval;
 
     switch (color) {
     case PBM_WHITE:
@@ -303,21 +307,38 @@ g3code(unsigned int const curcode,
 }
 
 
+
+enum g3tableId {TERMWHITE, TERMBLACK, MKUPWHITE, MKUPBLACK};
+
+
+
 static void
-processG3Code(tableentry *   const teP,
+processG3Code(g3TableEntry * const teP,
               bit *          const bitrow,
               unsigned int * const colP,
               bit *          const colorP,
               unsigned int * const countP) {
+              
+    enum g3tableId const teId =
+        (teP > mtable ? 2 : 0) + (teP - ttable) % 2;
 
-    switch (teP->tabid) {
-    case TWTABLE:
-    case TBTABLE: {
+    unsigned int teCount;
+    
+    switch(teId) {
+    case TERMWHITE: teCount = (teP - ttable    ) / 2;      break;
+    case TERMBLACK: teCount = (teP - ttable - 1) / 2;      break;
+    case MKUPWHITE: teCount = (teP - mtable    ) / 2 * 64; break;
+    case MKUPBLACK: teCount = (teP - mtable - 1) / 2 * 64; break;
+    }
+
+    switch (teId) {
+    case TERMWHITE:
+    case TERMBLACK: {
         unsigned int runLengthSoFar;
         unsigned int col;
         
         col = *colP;
-        runLengthSoFar = MIN(*countP + teP->count, MAXCOLS-col);
+        runLengthSoFar = MIN(*countP + teCount, MAXCOLS - col);
 
         if (runLengthSoFar > 0) {
             if (*colorP == PBM_WHITE) {
@@ -331,19 +352,14 @@ processG3Code(tableentry *   const teP,
         }
         *colorP = !*colorP;
         *countP = 0;
-        *colP = col;
-    }
-        break;
-    case MWTABLE:
-    case MBTABLE:
-        *countP += teP->count;
-        break;
-    case EXTABLE:
-        *countP += teP->count;
+        *colP   = col;
+    } break;
+    case MKUPWHITE:
+    case MKUPBLACK:
+        *countP += teCount;
         break;
     default:
-        pm_error("INTERNAL ERROR: invalid value %d for 'tabid' field "
-                 "of a tableentry in our hash", teP->tabid);
+        pm_error("Can't happen");
     }
 }
 
@@ -397,7 +413,7 @@ readFaxRow(struct bitStream * const bitStreamP,
         /* Number of consecutive pixels of the same color */
     bit currentColor;
         /* The color of the current run of pixels */
-    tableentry * te;
+    g3TableEntry * te;
         /* Address of structure that describes the current G3 code */
     bool done;
 
