@@ -27,23 +27,50 @@
 
 enum colortype {TRUECOLOR, PALETTE};
 
-struct cmdlineInfo {
-    /* All the information the user supplied in the command line,
-       in a form easy for the program to use.
-    */
-    const char * inputFilename;
-    int          class;  /* C_WIN or C_OS2 */
-    unsigned int bppSpec;
-    unsigned int bpp;
-};
+typedef struct {
+/*----------------------------------------------------------------------------
+   A color map for a BMP file.
+-----------------------------------------------------------------------------*/
+    unsigned int count;
+        /* Number of colors in the map.  The first 'count' elements of these
+           arrays are defined; all others are not.
+        */
+    colorhash_table cht;
+
+    /* Indices in the following arrays are the same as in 'cht', above. */
+    unsigned char red[MAXCOLORS];
+    unsigned char grn[MAXCOLORS];
+    unsigned char blu[MAXCOLORS];
+} colorMap;
+
 
 
 static void
-parseCommandLine(int argc, char ** argv,
-                 struct cmdlineInfo * const cmdlineP) {
+freeColorMap(const colorMap * const colorMapP) {
+
+    if (colorMapP->cht)
+        ppm_freecolorhash(colorMapP->cht);
+}
+
+
+
+static struct cmdline_info {
+    /* All the information the user supplied in the command line,
+       in a form easy for the program to use.
+    */
+    char *input_filename;
+    int class;  /* C_WIN or C_OS2 */
+    unsigned int bppSpec;
+    unsigned int bpp;
+} cmdline;
+
+
+static void
+parse_command_line(int argc, char ** argv,
+                   struct cmdline_info *cmdline_p) {
 /*----------------------------------------------------------------------------
    Note that many of the strings that this function returns in the
-   *cmdlineP structure are actually in the supplied argv array.  And
+   *cmdline_p structure are actually in the supplied argv array.  And
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
@@ -59,8 +86,8 @@ parseCommandLine(int argc, char ** argv,
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3('w', "windows",   OPT_FLAG, NULL, &windowsSpec,            0);
     OPTENT3('o', "os2",       OPT_FLAG, NULL, &os2Spec,                0);
-    OPTENT3(0,   "bpp",       OPT_UINT, &cmdlineP->bpp, 
-            &cmdlineP->bppSpec,      0);
+    OPTENT3(0,   "bpp",       OPT_UINT, &cmdline_p->bpp, 
+            &cmdline_p->bppSpec,      0);
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -71,28 +98,28 @@ parseCommandLine(int argc, char ** argv,
     if (windowsSpec && os2Spec) 
         pm_error("Can't specify both -windows and -os2 options.");
     else if (windowsSpec) 
-        cmdlineP->class = C_WIN;
+        cmdline_p->class = C_WIN;
     else if (os2Spec)
-        cmdlineP->class = C_OS2;
+        cmdline_p->class = C_OS2;
     else 
-        cmdlineP->class = C_WIN;
+        cmdline_p->class = C_WIN;
 
 
-    if (cmdlineP->bppSpec) {
-        if (cmdlineP->bpp != 1 && cmdlineP->bpp != 4 && 
-            cmdlineP->bpp != 8 && cmdlineP->bpp != 24)
+    if (cmdline_p->bppSpec) {
+        if (cmdline_p->bpp != 1 && cmdline_p->bpp != 4 && 
+            cmdline_p->bpp != 8 && cmdline_p->bpp != 24)
         pm_error("Invalid -bpp value specified: %u.  The only values valid\n"
                  "in the BMP format are 1, 4, 8, and 24 bits per pixel",
-                 cmdlineP->bpp);
+                 cmdline_p->bpp);
     }
 
     if (argc - 1 == 0)
-        cmdlineP->inputFilename = strdup("-");  /* he wants stdin */
+        cmdline_p->input_filename = strdup("-");  /* he wants stdin */
     else if (argc - 1 == 1)
-        cmdlineP->inputFilename = strdup(argv[1]);
+        cmdline_p->input_filename = strdup(argv[1]);
     else 
         pm_error("Too many arguments.  The only argument accepted\n"
-                 "is the input file specification");
+                 "is the input file specificaton");
 
 }
 
@@ -245,27 +272,27 @@ BMPwritergb(FILE * const fp,
 
 
 static int
-BMPwritecolormap(FILE *        const fp, 
-                 int           const class, 
-                 int           const bpp,
-                 int           const colors, 
-                 unsigned char const R[], 
-                 unsigned char const G[], 
-                 unsigned char const B[]) {
+BMPwritecolormap(FILE *           const ifP, 
+                 int              const class, 
+                 int              const bpp,
+                 const colorMap * const colorMapP) {
 /*----------------------------------------------------------------------------
   Return the number of bytes written, or -1 on error.
 -----------------------------------------------------------------------------*/
     long const ncolors = (1 << bpp);
 
-    int  nbyte;
-    int  i;
+    unsigned int  nbyte;
+    unsigned int  i;
 
     nbyte = 0;
-    for (i = 0; i < colors; ++i)
-        nbyte += BMPwritergb(fp,class,R[i],G[i],B[i]);
+    for (i = 0; i < colorMapP->count; ++i)
+        nbyte += BMPwritergb(ifP, class,
+                             colorMapP->red[i],
+                             colorMapP->grn[i],
+                             colorMapP->blu[i]);
 
     for (; i < ncolors; ++i)
-        nbyte += BMPwritergb(fp,class,0,0,0);
+        nbyte += BMPwritergb(ifP, class, 0, 0, 0);
 
     return nbyte;
 }
@@ -273,11 +300,11 @@ BMPwritecolormap(FILE *        const fp,
 
 
 static int
-BMPwriterowPalette(FILE *          const fp, 
-                   const pixel *   const row, 
-                   unsigned long   const cx, 
-                   unsigned short  const bpp, 
-                   colorhash_table const cht) {
+BMPwriterow_palette(FILE *          const fp, 
+                    const pixel *   const row, 
+                    unsigned long   const cx, 
+                    unsigned short  const bpp, 
+                    colorhash_table const cht) {
 /*----------------------------------------------------------------------------
   Return the number of bytes written, or -1 on error.
 -----------------------------------------------------------------------------*/
@@ -329,10 +356,10 @@ BMPwriterowPalette(FILE *          const fp,
 
 
 static int
-BMPwriterowTruecolor(FILE *        const fp, 
-                     const pixel * const row, 
-                     unsigned long const cols,
-                     pixval        const maxval) {
+BMPwriterow_truecolor(FILE *        const fp, 
+                      const pixel * const row, 
+                      unsigned long const cols,
+                      pixval        const maxval) {
 /*----------------------------------------------------------------------------
   Write a row of a truecolor BMP image to the file 'fp'.  The row is 
   'row', which is 'cols' columns long.
@@ -345,7 +372,7 @@ BMPwriterowTruecolor(FILE *        const fp,
        general case (which is only hypothetical -- this program doesn't
        write any truecolor images except 24 bit and apparently no one
        else does either), you would move this function into 
-       BMPwriterowPalette, which writes arbitrary bit strings.  But
+       BMPwriterow_palette, which writes arbitrary bit strings.  But
        that would be a lot slower and less robust.
     */
 
@@ -399,10 +426,10 @@ BMPwritebits(FILE *          const fp,
     for (y = cy - 1; y >= 0; --y) {
         int rc;
         if (colortype == PALETTE)
-            rc = BMPwriterowPalette(fp, pixels[y], cx, 
-                                    cBitCount, cht);
+            rc = BMPwriterow_palette(fp, pixels[y], cx, 
+                                     cBitCount, cht);
         else 
-            rc = BMPwriterowTruecolor(fp, pixels[y], cx, maxval);
+            rc = BMPwriterow_truecolor(fp, pixels[y], cx, maxval);
 
         if (rc == -1)
             pm_error("couldn't write row %ld", y);
@@ -417,25 +444,17 @@ BMPwritebits(FILE *          const fp,
 
 
 static void
-BMPEncode(FILE *          const fp, 
-          int             const class, 
-          enum colortype  const colortype,
-          int             const bpp,
-          int             const x, 
-          int             const y, 
-          const pixel **  const pixels, 
-          pixval          const maxval,
-          int             const colors, 
-          colorhash_table const cht, 
-          unsigned char   const R[], 
-          unsigned char   const G[], 
-          unsigned char   const B[]) {
+BMPEncode(FILE *           const ifP, 
+          int              const class, 
+          enum colortype   const colortype,
+          int              const bpp,
+          int              const x, 
+          int              const y, 
+          const pixel **   const pixels, 
+          pixval           const maxval,
+          const colorMap * const colorMapP) {
 /*----------------------------------------------------------------------------
   Write a BMP file of the given class.
-  
-  Note that we must have 'colors' in order to know exactly how many
-  colors are in the R, G, B, arrays.  Entries beyond those in the
-  arrays are undefined.
 -----------------------------------------------------------------------------*/
     unsigned long nbyte;
 
@@ -445,17 +464,18 @@ BMPEncode(FILE *          const fp,
         pm_message("Writing %d bits per pixel truecolor (no palette)", bpp);
 
     nbyte = 0;  /* initial value */
-    nbyte += BMPwritefileheader(fp, class, bpp, x, y);
-    nbyte += BMPwriteinfoheader(fp, class, bpp, x, y);
+    nbyte += BMPwritefileheader(ifP, class, bpp, x, y);
+    nbyte += BMPwriteinfoheader(ifP, class, bpp, x, y);
     if (colortype == PALETTE)
-        nbyte += BMPwritecolormap(fp, class, bpp, colors, R, G, B);
+        nbyte += BMPwritecolormap(ifP, class, bpp, colorMapP);
 
     if (nbyte != (BMPlenfileheader(class)
                   + BMPleninfoheader(class)
                   + BMPlencolormap(class, bpp, -1)))
         pm_error(er_internal, "BMPEncode 1");
 
-    nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, maxval, cht);
+    nbyte += BMPwritebits(ifP, x, y, colortype, bpp, pixels, maxval,
+                          colorMapP->cht);
     if (nbyte != BMPlenfile(class, bpp, -1, x, y))
         pm_error(er_internal, "BMPEncode 2");
 }
@@ -463,50 +483,120 @@ BMPEncode(FILE *          const fp,
 
 
 static void
-analyzeColors(const pixel **    const pixels, 
-              int               const cols, 
-              int               const rows, 
-              pixval            const maxval, 
-              int *             const colorsP, 
-              int *             const minimumBppP,
-              colorhash_table * const chtP, 
-              unsigned char           Red[], 
-              unsigned char           Green[], 
-              unsigned char           Blue[]) {
+makeBilevelColorMap(colorMap * const colorMapP) {
+
+    colorMapP->count  = 2;
+    colorMapP->cht    = NULL;
+    colorMapP->red[0] = 0;
+    colorMapP->grn[0] = 0;
+    colorMapP->blu[0] = 0;
+    colorMapP->red[1] = 255;
+    colorMapP->grn[1] = 255;
+    colorMapP->blu[1] = 255;
+}
+
+
+
+static void
+BMPEncodePBM(FILE *           const ifP, 
+             int              const class, 
+             int              const cols, 
+             int              const rows, 
+             unsigned char ** const bitrow) {
+/*----------------------------------------------------------------------------
+  Write a bi-level BMP file of the given class.
+-----------------------------------------------------------------------------*/
+    /* Note:
+       Only PBM input uses this routine.  Color images represented by 1 bpp via
+       color palette use the general BMPEncode().
+    */
+    unsigned int const adjustedCols = (cols + 31) / 32 * 32;
+    unsigned int const packedBytes  = adjustedCols / 8;
+
+    unsigned long nbyte;
+    colorMap bilevelColorMap;
+    unsigned int row;
+    
+    /* colortype == PALETTE */
+    pm_message("Writing 1 bit per pixel with a black-white palette");
+
+    nbyte = 0;  /* initial value */
+    nbyte += BMPwritefileheader(ifP, class, 1, cols, rows);
+    nbyte += BMPwriteinfoheader(ifP, class, 1, cols, rows);
+
+    makeBilevelColorMap(&bilevelColorMap);
+
+    nbyte += BMPwritecolormap(ifP, class, 2, &bilevelColorMap);
+
+    if (nbyte != (BMPlenfileheader(class)
+                  + BMPleninfoheader(class)
+                  + BMPlencolormap(class, 1, -1)))
+        pm_error(er_internal, "BMPEncodePBM 1");
+   
+    for (row = 0; row < rows; ++row){
+        size_t bytesWritten;
+
+        bytesWritten = fwrite(bitrow[row], 1, packedBytes, ifP);
+        if (bytesWritten != packedBytes){
+            if (feof(ifP))
+                pm_error("End of file writing row %u of BMP raster.", row);
+            else 
+                pm_error("Error writing BMP raster.  Errno=%d (%s)",
+                         errno, strerror(errno));
+        }  else
+            nbyte += bytesWritten;
+    }
+
+    if (nbyte != BMPlenfile(class, 1, -1, cols, rows))
+        pm_error(er_internal, "BMPEncodePBM 2");
+}
+
+
+
+static void
+analyze_colors(const pixel **    const pixels, 
+               int               const cols, 
+               int               const rows, 
+               pixval            const maxval, 
+               int *             const minimum_bpp_p,
+               colorMap *        const colorMapP) {
 /*----------------------------------------------------------------------------
   Look at the colors in the image 'pixels' and compute values to use in
   representing those colors in a BMP image.  
 
-  First of all, count the distinct colors.  Return as *minimumBppP
+  First of all, count the distinct colors.  Return as *minimum_bpp_p
   the minimum number of bits per pixel it will take to represent all
   the colors in BMP format.
 
   If there are few enough colors to represent with a palette, also
-  return the number of colors as *colorsP and a suitable palette
+  return the number of colors as *colors_p and a suitable palette
   (colormap) and a hash table in which to look up indexes into that
-  palette as Red[], Green[], Blue[], and *chtP.  Use only the first
-  *colorsP entries of the Red[], etc. array.
+  palette as *colorMapP.  Use only the first *colors_p entries of the
+  map.
 
-  If there are too many colors for a palette, return *chtP == NULL.
+  If there are too many colors for a palette, return colorMapP->cht
+  == NULL.
 
   Issue informational messages.
 -----------------------------------------------------------------------------*/
-
     /* Figure out the colormap. */
     colorhist_vector chv;
-    int i;
+    int colorCount;
 
     pm_message("analyzing colors...");
     chv = ppm_computecolorhist((pixel**)pixels, cols, rows, MAXCOLORS, 
-                               colorsP);
+                               &colorCount);
+    colorMapP->count = colorCount;
     if (chv == NULL) {
-        pm_message("More than %d colors found", MAXCOLORS);
-        *minimumBppP = 24;
-        *chtP = NULL;
+        pm_message("More than %u colors found", MAXCOLORS);
+        *minimum_bpp_p = 24;
+        colorMapP->cht = NULL;
     } else {
-        unsigned int const minbits = pm_maxvaltobits(*colorsP-1);
+        unsigned int const minbits = pm_maxvaltobits(colorMapP->count - 1);
 
-        pm_message("%d colors found", *colorsP);
+        unsigned int i;
+
+        pm_message("%u colors found", colorMapP->count);
 
         /* Only 1, 4, 8, and 24 are defined in the BMP spec we
            implement and other bpp's have in fact been seen to confuse
@@ -515,25 +605,25 @@ analyzeColors(const pixel **    const pixels,
            (see Bmptopnm.c, though).  
         */
         if (minbits == 1)
-            *minimumBppP = 1;
+            *minimum_bpp_p = 1;
         else if (minbits <= 4)
-            *minimumBppP = 4;
+            *minimum_bpp_p = 4;
         else if (minbits <= 8)
-            *minimumBppP = 8;
+            *minimum_bpp_p = 8;
         else
-            *minimumBppP = 24;
+            *minimum_bpp_p = 24;
 
         /*
          * Now scale the maxval to 255 as required by BMP format.
          */
-        for (i = 0; i < *colorsP; ++i) {
-            Red[i] = (pixval) PPM_GETR(chv[i].color) * 255 / maxval;
-            Green[i] = (pixval) PPM_GETG(chv[i].color) * 255 / maxval;
-            Blue[i] = (pixval) PPM_GETB(chv[i].color) * 255 / maxval;
+        for (i = 0; i < colorMapP->count; ++i) {
+            colorMapP->red[i] = (pixval) PPM_GETR(chv[i].color) * 255 / maxval;
+            colorMapP->grn[i] = (pixval) PPM_GETG(chv[i].color) * 255 / maxval;
+            colorMapP->blu[i] = (pixval) PPM_GETB(chv[i].color) * 255 / maxval;
         }
     
         /* And make a hash table for fast lookup. */
-        *chtP = ppm_colorhisttocolorhash(chv, *colorsP);
+        colorMapP->cht = ppm_colorhisttocolorhash(chv, colorMapP->count);
         ppm_freecolorhist(chv);
     }
 }
@@ -541,19 +631,19 @@ analyzeColors(const pixel **    const pixels,
 
 
 static void
-chooseColortypeBpp(struct cmdlineInfo const cmdline,
-                   int                 const colors, 
-                   int                 const minimumBpp,
-                   enum colortype *    const colortypeP, 
-                   int *               const bitsPerPixelP) {
+choose_colortype_bpp(struct cmdline_info const cmdline,
+                     unsigned int        const colors, 
+                     unsigned int        const minimum_bpp,
+                     enum colortype *    const colortype_p, 
+                     unsigned int *      const bits_per_pixel_p) {
 
     if (!cmdline.bppSpec) {
         /* User has no preference as to bits per pixel.  Choose the
            smallest number possible for this image.
         */
-        *bitsPerPixelP = minimumBpp;
+        *bits_per_pixel_p = minimum_bpp;
     } else {
-        if (cmdline.bpp < minimumBpp)
+        if (cmdline.bpp < minimum_bpp)
             pm_error("There are too many colors in the image to "
                      "represent in the\n"
                      "number of bits per pixel you requested: %d.\n"
@@ -561,18 +651,18 @@ chooseColortypeBpp(struct cmdlineInfo const cmdline,
                      "colors in the image.",
                      cmdline.bpp);
         else
-            *bitsPerPixelP = cmdline.bpp;
+            *bits_per_pixel_p = cmdline.bpp;
     }
 
-    assert(*bitsPerPixelP == 1 || 
-           *bitsPerPixelP == 4 || 
-           *bitsPerPixelP == 8 || 
-           *bitsPerPixelP == 24);
+    assert(*bits_per_pixel_p == 1 || 
+           *bits_per_pixel_p == 4 || 
+           *bits_per_pixel_p == 8 || 
+           *bits_per_pixel_p == 24);
 
-    if (*bitsPerPixelP > 8) 
-        *colortypeP = TRUECOLOR;
+    if (*bits_per_pixel_p > 8) 
+        *colortype_p = TRUECOLOR;
     else {
-        *colortypeP = PALETTE;
+        *colortype_p = PALETTE;
     }
 }
 
@@ -581,48 +671,105 @@ chooseColortypeBpp(struct cmdlineInfo const cmdline,
 int
 main(int argc, char **argv) {
 
-    struct cmdlineInfo cmdline;
     FILE *ifp;
+
     int rows;
     int cols;
-    int colors;
+    int row;
     pixval maxval;
-    int minimumBpp;
-    int bitsPerPixel;
-    enum colortype colortype;
+    int ppmFormat;
+
+
 
-    pixel** pixels;
-    colorhash_table cht;
-    /* 'Red', 'Green', and 'Blue' are the color map for the BMP file,
-       with indices the same as in 'cht', above.
-    */
-    unsigned char Red[MAXCOLORS], Green[MAXCOLORS], Blue[MAXCOLORS];
 
     ppm_init(&argc, argv);
 
-    parseCommandLine(argc, argv, &cmdline);
+    parse_command_line(argc, argv, &cmdline);
 
-    ifp = pm_openr(cmdline.inputFilename);
+    ifp = pm_openr(cmdline.input_filename);
+    
+    /*     pixels = ppm_readppm(ifp, &cols, &rows, &maxval) ; */
+    
+    ppm_readppminit(ifp, &cols, &rows, &maxval, &ppmFormat);
+    
+    if(ppmFormat == PBM_FORMAT || ppmFormat == RPBM_FORMAT) {
+        /*  In the PBM case the raster is read directly from the input by 
+            pbm_readpbmrow_packed.  The raster format is almost identical,
+            except that BMP specifies rows to be zero-filled to 32 bit borders 
+            and that in BMP the bottom row comes first in order.
+        */
 
-    pixels = ppm_readppm(ifp, &cols, &rows, &maxval);
+        unsigned char ** bitrow;    
+        int const CHARBITS = (sizeof(unsigned char)*8); 
+        int const colChars = pbm_packed_bytes(cols);
+        int const adjustedCols = (cols+31) /32 * 32;
+        int const packedBytes  =  adjustedCols /8;
 
-    pm_close(ifp);
+        int i,r;
 
+        bitrow = pbm_allocarray_packed(adjustedCols, rows);
 
-    analyzeColors((const pixel**)pixels, cols, rows, maxval, 
-                   &colors, &minimumBpp, &cht, Red, Green, Blue);
+        for (r=0 ; r < rows; ++r){
+            row = rows -r -1;
+            /* Clear end of each row */
+            bitrow[row][packedBytes-1] = bitrow[row][packedBytes-2] =
+                bitrow[row][packedBytes-3] = bitrow[row][packedBytes-4] = 0x00;
 
-    chooseColortypeBpp(cmdline, colors, minimumBpp, &colortype, 
-                       &bitsPerPixel);
+            pbm_readpbmrow_packed(ifp, bitrow[row], cols, ppmFormat);
 
-    BMPEncode(stdout, cmdline.class, colortype, bitsPerPixel,
-              cols, rows, (const pixel**)pixels, maxval, colors, cht,
-              Red, Green, Blue);
+            for(i=0; i<colChars; ++i) 
+                bitrow[row][i] = ~ bitrow[row][i]; /* flip all pixels */
+ 
+            /* This may seem unnecessary, because the color palette 
+               (RGB[] in BMPEncodePBM) can be inverted for the same effect.
+               However we take this precaution, for there is indication that
+               some BMP viewers may get confused with that.
+            */
 
-    if (cht) 
-        ppm_freecolorhash(cht);
+            if (cols %8 >0){ /* adjust final partial byte */
+                bitrow[row][colChars-1] >>= CHARBITS - cols % CHARBITS;
+                bitrow[row][colChars-1] <<= CHARBITS - cols % CHARBITS;
+            }
+        }
+        pm_close(ifp);
 
-    pm_close(stdout);
+        BMPEncodePBM(stdout, cmdline.class, cols, rows, bitrow);
 
-    return 0;
+        pm_close(stdout);
+        return 0;
+    }            
+        
+    else {
+        /* PGM and PPM.  The input image is read into a PPM array, scanned
+           for color analysis and converted to a BMP raster.
+           Logic works for PBM.
+        */
+        int minimum_bpp;
+        unsigned int bits_per_pixel;
+        enum colortype colortype;
+       
+        pixel** pixels;
+        colorMap colorMap;
+       
+        pixels = ppm_allocarray(cols, rows);
+
+        for (row = 0; row < rows; ++row)
+            ppm_readppmrow(ifp, pixels[row], cols, maxval, ppmFormat);
+
+        pm_close(ifp);
+
+        analyze_colors((const pixel**)pixels, cols, rows, maxval, 
+                       &minimum_bpp, &colorMap);
+
+        choose_colortype_bpp(cmdline, colorMap.count, minimum_bpp, &colortype, 
+                             &bits_per_pixel);
+
+        BMPEncode(stdout, cmdline.class, colortype, bits_per_pixel,
+                  cols, rows, (const pixel**)pixels, maxval, &colorMap);
+
+        freeColorMap(&colorMap);
+        
+        pm_close(stdout);
+        return 0;
+    }
 }
