@@ -38,11 +38,12 @@
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/times.h>
-#include <sys/utsname.h>
 
 #include "mallocvar.h"
 #include "nstring.h"
 
+#include "pm.h"
+
 #include "all.h"
 #include "param.h"
 #include "mpeg.h"
@@ -54,6 +55,8 @@
 #include "input.h"
 #include "psocket.h"
 #include "frametype.h"
+#include "gethostname.h"
+
 #include "parallel.h"
 
 
@@ -136,25 +139,6 @@ int     ClientPid[MAX_MACHINES+4];
  *=====================*/
 
 
-static const char *
-getHostName(void) {
-/*----------------------------------------------------------------------------
-   Return the host name of this system.
------------------------------------------------------------------------------*/
-    struct utsname utsname;
-    int rc;
-
-    rc = uname(&utsname);
-
-    if (rc < 0)
-        pm_error("Unable to find out host name.  "
-                 "uname() failed with errno %d (%s)", errno, strerror(errno));
-
-    return strdup(utsname.nodename);
-}
-
-
-
 static void GNU_PRINTF_ATTR
 machineDebug(const char format[], ...) {
 
@@ -163,7 +147,7 @@ machineDebug(const char format[], ...) {
     va_start(args, format);
 
     if (debugMachines) {
-        const char * const hostname = getHostName();
+        const char * const hostname = GetHostName();
         fprintf(stderr, "%s: ---", hostname);
         strfree(hostname);
         vfprintf(stderr, format, args);
@@ -177,7 +161,7 @@ machineDebug(const char format[], ...) {
 static void GNU_PRINTF_ATTR
 errorExit(const char format[], ...) {
 
-    const char * const hostname = getHostName();
+    const char * const hostname = GetHostName();
 
     va_list args;
 
@@ -1070,6 +1054,7 @@ GetRemoteFrame(MpegFrame * const frameP,
             } while ( numBytes == sizeof(buffer) );
             fflush(filePtr);
             rewind(filePtr);
+            /* now do slave conversion */
             ReadFrameFile(frameP, filePtr, slaveConversion);
             fclose(filePtr);
         } else {
@@ -1962,7 +1947,7 @@ MasterServer(struct inputSource * const inputSourceP,
 
     PrintStartStats(startUpBegin, FALSE, 0, 0, inputSourceP);
 
-    hostName = getHostName();
+    hostName = GetHostName();
 
     hostEntry = gethostbyname(hostName);
     if (hostEntry == NULL)
