@@ -1596,6 +1596,11 @@ horizontalScale(tuplen *     const inputtuplenrow,
   it by a factor of 'xscale', to create the output row 'newtuplenrow',
   described by *outpamP.
 
+  Due to arithmetic imprecision, we may have to stretch slightly the
+  contents of the last pixel of the output row to make a full pixel.
+  Return as *stretchP the fraction of a pixel by which we had to
+  stretch in this way.
+
   Assume maxval and depth of input and output are the same.
 -----------------------------------------------------------------------------*/
     float fraccoltofill, fraccolleft;
@@ -1652,7 +1657,8 @@ horizontalScale(tuplen *     const inputtuplenrow,
         /* We were still working on the last output column when we 
            ran out of input columns.  This would be because of rounding
            down, and we should be missing only a tiny fraction of that
-           last output column.
+           last output column.  Just fill in the missing color with the
+           color of the rightmost input pixel.
         */
         accumOutputCol(inpamP, inputtuplenrow[inpamP->width-1], 
                        fraccoltofill, newtuplenrow[newcol]);
