@@ -25,10 +25,11 @@
 #include <math.h>
 #include <assert.h>
 
-#include "pam.h"
-#include "pammap.h"
 #include "shhopt.h"
 #include "mallocvar.h"
+#include "nstring.h"
+#include "pam.h"
+#include "pammap.h"
 
 #define MAXCOLORS 32767u
 
@@ -81,7 +82,7 @@ parseCommandLine (int argc, char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry * option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -90,6 +91,8 @@ parseCommandLine (int argc, char ** argv,
 
     unsigned int nofloyd, firstisdefault;
     unsigned int missingSpec, mapfileSpec;
+
+    MALLOCARRAY_NOFAIL(option_def, 100);
     
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "floyd",        OPT_FLAG,   
