@@ -63,7 +63,7 @@ parseCommandLine(int argc, char ** argv,
         cmdlineP->inputFilename = argv[2];
     else 
         pm_error("Too many arguments.  The only arguments accepted "
-                 "are the mask color and optional input file specificaton");
+                 "are the mask color and optional input file specification");
 }
 
 
