@@ -32,59 +32,6 @@
 
 #include "libpam.h"
 
-char
-stripeq(const char * const comparand, const char * const comparator) {
-/*----------------------------------------------------------------------------
-  Compare two strings, ignoring leading and trailing white space.
-
-  Return true (1) if the strings are identical, false (0) otherwise.
------------------------------------------------------------------------------*/
-
-  char *p, *q, *px, *qx;
-  char equal;
-  
-  /* Make p and q point to the first non-blank character in each string.
-     If there are no non-blank characters, make them point to the terminating
-     NULL.
-     */
-
-  p = (char *) comparand;
-  while (ISSPACE(*p)) p++;
-  q = (char *) comparator;
-  while (ISSPACE(*q)) q++;
-
-  /* Make px and qx point to the last non-blank character in each string.
-     If there are no nonblank characters (which implies the string is
-     null), make them point to the terminating NULL.
-     */
-
-  if (*p == '\0') px = p;
-  else {
-    px = p + strlen(p) - 1;
-    while (ISSPACE(*px)) px--;
-  }
-
-  if (*q == '\0') qx = q;
-  else {
-    qx = q + strlen(q) - 1;
-    while (ISSPACE(*qx)) qx--;
-  }
-
-  equal = TRUE;   /* initial assumption */
-  
-  /* If the stripped strings aren't the same length, 
-     we know they aren't equal 
-     */
-  if (px - p != qx - q) equal = FALSE;
-
-
-  while (p <= px) {
-    if (*p != *q) equal = FALSE;
-    p++; q++;
-  }
-  return equal;
-}
-
 
 
 static unsigned int
@@ -363,7 +310,13 @@ pnm_scaletuplerow(const struct pam * const pamP,
                   sample             const newMaxval) {
 
     if (pamP->maxval == newMaxval) {
-        /* Fast path for common case:  It's already what it needs to be. */
+        /* Fast path for common case: no scaling needed */
+        if (destRow != sourceRow) {
+            /* Fast path for common case: it's already what it needs to be */
+            unsigned int col;
+            for (col = 0; col < pamP->width; ++col)
+                pnm_assigntuple(pamP, destRow[col], sourceRow[col]);
+        }
     } else {        
         unsigned int col;
         for (col = 0; col < pamP->width; ++col) 
@@ -673,7 +626,6 @@ pnm_readpaminitrestaspnm(FILE * const fileP,
     pam.file        = fileP;
     pam.len         = sizeof(struct pam);
     pam.format      = PAM_FORMAT;
-    pam.plainformat = 0;
 
     readpaminitrest(&pam);
 
@@ -741,11 +693,6 @@ pnm_readpaminit(FILE *       const file,
     /* Get magic number. */
     pamP->format = pm_readmagicnumber(file);
 
-    pamP->plainformat =
-        (pamP->format == PBM_FORMAT || 
-         pamP->format == PGM_FORMAT ||
-         pamP->format == PPM_FORMAT);
-
     switch (PAM_FORMAT_TYPE(pamP->format)) {
     case PAM_TYPE: 
         readpaminitrest(pamP);
@@ -819,6 +766,9 @@ pnm_writepaminit(struct pam * const pamP) {
     
     switch (PAM_FORMAT_TYPE(pamP->format)) {
     case PAM_TYPE:
+        if (pm_plain_output)
+            pm_error("There is no plain version of PAM.  -plain option "
+                     "is not allowed");
         fprintf(pamP->file, "P7\n");
         fprintf(pamP->file, "WIDTH %u\n",   (unsigned)pamP->width);
         fprintf(pamP->file, "HEIGHT %u\n",  (unsigned)pamP->height);
