@@ -692,6 +692,33 @@ pm_openw(const char * const name) {
 
 
 
+static const char *
+tmpDir(void) {
+/*----------------------------------------------------------------------------
+   Return the name of the directory in which we should create temporary
+   files.
+
+   The name is a constant in static storage.
+-----------------------------------------------------------------------------*/
+    const char * tmpdir;
+        /* running approximation of the result */
+
+    tmpdir = getenv("TMPDIR");   /* Unix convention */
+
+    if (!tmpdir || strlen(tmpdir) == 0)
+        tmpdir = getenv("TMP");  /* Windows convention */
+
+    if (!tmpdir || strlen(tmpdir) == 0)
+        tmpdir = getenv("TEMP"); /* Windows convention */
+
+    if (!tmpdir || strlen(tmpdir) == 0)
+        tmpdir = TMPDIR;
+
+    return tmpdir;
+}
+
+
+
 void
 pm_make_tmpfile(FILE **       const filePP,
                 const char ** const filenameP) {
@@ -700,22 +727,13 @@ pm_make_tmpfile(FILE **       const filePP,
     FILE * fileP;
     const char * filenameTemplate;
     char * filenameBuffer;  /* malloc'ed */
-    const char * tbuf;
     unsigned int fnamelen;
     const char * tmpdir;
     const char * dirseparator;
 
     fnamelen = strlen (pm_progname) + 10; /* "/" + "_XXXXXX\0" */
 
-    tbuf = getenv("TMPDIR");
-
-    if ((tbuf == NULL) || (strlen(tbuf) == 0))
-        /* environment variable not suitable to construct file name.
-           Use default.
-        */
-        tmpdir = TMPDIR;
-    else
-        tmpdir = tbuf;
+    tmpdir = tmpDir();
 
     if (tmpdir[strlen(tmpdir) - 1] == '/')
         dirseparator = "";
@@ -732,8 +750,9 @@ pm_make_tmpfile(FILE **       const filePP,
     fd = mkstemp(filenameBuffer);
 
     if (fd < 0)
-        pm_error("Unable to create temporary file.  mkstemp() failed with "
-                 "errno %d (%s)", errno, strerror(errno));
+        pm_error("Unable to create temporary file according to name "
+                 "pattern '%s'.  mkstemp() failed with "
+                 "errno %d (%s)", filenameBuffer, errno, strerror(errno));
     else {
         fileP = fdopen(fd, "w+b");
 
@@ -1141,6 +1160,45 @@ pm_read_unknown_size(FILE * const file,
 
 
 
+union cheat {
+    uint32n l;
+    short s;
+    unsigned char c[4];
+};
+
+
+
+short
+pm_bs_short(short const s) {
+    union cheat u;
+    unsigned char t;
+
+    u.s = s;
+    t = u.c[0];
+    u.c[0] = u.c[1];
+    u.c[1] = t;
+    return u.s;
+}
+
+
+
+long
+pm_bs_long(long const l) {
+    union cheat u;
+    unsigned char t;
+
+    u.l = l;
+    t = u.c[0];
+    u.c[0] = u.c[3];
+    u.c[3] = t;
+    t = u.c[1];
+    u.c[1] = u.c[2];
+    u.c[2] = t;
+    return u.l;
+}
+
+
+
 void
 pm_tell2(FILE *       const fileP, 
          void *       const fileposP,
