@@ -283,6 +283,12 @@ char*
 pm_read_unknown_size(FILE * const ifP, 
                      long * const buf);
 
+short
+pm_bs_short(short const s);
+
+long
+pm_bs_long(long const l);
+
 unsigned int
 pm_tell(FILE * const fileP);
 
