@@ -1,5 +1,14 @@
 #ifndef PM_SYSTEM_H
 #define PM_SYSTEM_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+#if 0
+} /* to fake out automatic code indenters */
+#endif
+
+
 void
 pm_system(void                  stdinFeeder(int, void *),
           void *          const feederParm,
@@ -27,4 +36,8 @@ void
 pm_accept_to_memory(int    const pipetosuckFd,
                     void * const accepterParm );
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
