@@ -14,31 +14,16 @@ extern "C" {
 
 typedef gray pixval;
 
-#ifdef PPM_PACKCOLORS
+/* These are just for use in this header file */
+#define PPM_MAX(a,b) ((a) > (b) ? (a) : (b))
+#define PPM_MIN(a,b) ((a) < (b) ? (a) : (b))
 
-#define PPM_MAXMAXVAL 1023
-typedef unsigned long pixel;
-#define PPM_GETR(p) (((p) & 0x3ff00000) >> 20)
-#define PPM_GETG(p) (((p) & 0xffc00) >> 10)
-#define PPM_GETB(p) ((p) & 0x3ff)
-
-/************* added definitions *****************/
-#define PPM_PUTR(p, red) ((p) |= (((red) & 0x3ff) << 20))
-#define PPM_PUTG(p, grn) ((p) |= (((grn) & 0x3ff) << 10))
-#define PPM_PUTB(p, blu) ((p) |= ( (blu) & 0x3ff))
-/**************************************************/
-
-#define PPM_ASSIGN(p,red,grn,blu) (p) = ((pixel) (red) << 20) | ((pixel) (grn) << 10) | (pixel) (blu)
-#define PPM_EQUAL(p,q) ((p) == (q))
-
-#else /*PPM_PACKCOLORS*/
 
 #define PPM_OVERALLMAXVAL PGM_OVERALLMAXVAL
 #define PPM_MAXMAXVAL PGM_MAXMAXVAL
-typedef struct
-    {
+typedef struct {
     pixval r, g, b;
-    } pixel;
+} pixel;
 #define PPM_GETR(p) ((p).r)
 #define PPM_GETG(p) ((p).g)
 #define PPM_GETB(p) ((p).b)
@@ -54,8 +39,6 @@ typedef struct
 #define PPM_EQUAL(p,q) \
   ( (p).r == (q).r && (p).g == (q).g && (p).b == (q).b )
 
-#endif /*PPM_PACKCOLORS*/
-
 
 /* Magic constants. */
 
@@ -82,11 +65,12 @@ void ppm_init(int * argcP, char* argv[]);
 #define ppm_allocarray(cols, rows) \
   ((pixel**) pm_allocarray(cols, rows, sizeof(pixel)))
 
-#define ppm_allocrow(cols) ((pixel*) pm_allocrow(cols, sizeof(pixel)))
+pixel *
+ppm_allocrow(unsigned int const cols);
 
 #define ppm_freearray(pixels, rows) pm_freearray((char**) pixels, rows)
 
-#define ppm_freerow(pixelrow) pm_freerow((char*) pixelrow)
+#define ppm_freerow(pixelrow) free(pixelrow);
 
 pixel**
 ppm_readppm(FILE *   const fileP, 
@@ -144,7 +128,8 @@ ppm_nextimage(FILE * const fileP,
               int *  const eofP);
 
 pixel 
-ppm_parsecolor(const char* colorname, pixval maxval);
+ppm_parsecolor(const char * const colorname,
+               pixval       const maxval);
 
 pixel
 ppm_parsecolor2(const char * const colorname,
@@ -171,7 +156,7 @@ ppm_readcolornamefile(const char *      const fileName,
                       const char ***    const colornamesP);
 
 void
-ppm_freecolornames(const char ** colornames);
+ppm_freecolornames(const char ** const colornames);
 
 #define PPM_ISGRAY(pixel) \
     (PPM_GETR(pixel) == PPM_GETG(pixel) && PPM_GETR(pixel) == PPM_GETB(pixel))
@@ -184,10 +169,22 @@ ppm_freecolornames(const char ** colornames);
 	( (int) PPM_GETG(p) * (newmaxval) + (oldmaxval) / 2 ) / (oldmaxval), \
 	( (int) PPM_GETB(p) * (newmaxval) + (oldmaxval) / 2 ) / (oldmaxval) )
 
-#define PPM_DISTANCE(p1,p2) ( \
-  SQR(PPM_GETR(p1)-PPM_GETR(p2)) + \
-  SQR(PPM_GETG(p1)-PPM_GETR(p2)) + \
-  SQR(PPM_GETB(p1)-PPM_GETB(p2)) )
+#define PPM_SQR(x) (x)*(x)
+
+static __inline__ unsigned int
+PPM_DISTANCE(pixel const p1,
+             pixel const p2) {
+    return (
+        PPM_SQR(PPM_GETR(p1)-PPM_GETR(p2)) +
+        PPM_SQR(PPM_GETG(p1)-PPM_GETG(p2)) +
+        PPM_SQR(PPM_GETB(p1)-PPM_GETB(p2)) );
+}
+#undef PPM_SQR
+
+/* Note that because a sample can be at most 1 << 16 - 1, PPM_DISTANCE
+   is less than UINT_MAX.  That means you can use UINT_MAX as an infinite
+   distance in some applications.
+*/
 
 /* Luminance, Chrominance macros. */
 
@@ -241,7 +238,7 @@ ppm_colorvalue(pixel const p) {
 /*----------------------------------------------------------------------------
   The color value (V is HSV) as a pixval
 -----------------------------------------------------------------------------*/
-    return MAX(PPM_GETR(p), MAX(PPM_GETG(p), PPM_GETB(p)));
+    return PPM_MAX(PPM_GETR(p), PPM_MAX(PPM_GETG(p), PPM_GETB(p)));
 }
 
 static __inline pixval
@@ -252,9 +249,9 @@ ppm_saturation(pixel const p,
   is 100, this is 50).
 -----------------------------------------------------------------------------*/
     pixval const maxIntensity =
-        MAX(PPM_GETR(p), MAX(PPM_GETG(p), PPM_GETB(p)));
+        PPM_MAX(PPM_GETR(p), PPM_MAX(PPM_GETG(p), PPM_GETB(p)));
     pixval const minIntensity =
-        MIN(PPM_GETR(p), MIN(PPM_GETG(p), PPM_GETB(p)));
+        PPM_MIN(PPM_GETR(p), PPM_MIN(PPM_GETG(p), PPM_GETB(p)));
     pixval const range = maxIntensity - minIntensity;
 
     return (pixval)((unsigned long)range * maxval / maxIntensity);
@@ -264,5 +261,7 @@ ppm_saturation(pixel const p,
 }
 #endif
 
+#undef PPM_MIN
+#undef PPM_MAX
 
 #endif /*_PPM_H_*/
