@@ -3,6 +3,7 @@
 
 #include <stdarg.h>
 #include <string.h>
+#include <ctype.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -29,6 +30,9 @@ extern "C" {
     (strcasecmp((A), (B)) == 0)
 #define STRSEQ(A, B) \
 	(strncmp((A), (B), sizeof(A)) == 0)
+#define MEMSZERO(A) \
+    bzero((A), sizeof(A))
+
 
 
 /* The standard C library routines isdigit(), for some weird 
@@ -114,6 +118,10 @@ strfree(const char * const string);
 const char *
 strsepN(char ** const stringP, const char * const delim);
 
+int
+stripeq(const char * const comparand,
+        const char * const comparator);
+
 #ifdef __cplusplus
 }
 #endif
